@@ -1301,7 +1301,8 @@ class BatchOperations:
                 mutate_operations=self.mutate_operations
             )
 
-            self._print_response_details(response)
+            resource_names =self._print_response_details(response)
+            return resource_names
         else:
             print("No operations to run")
 
@@ -1434,6 +1435,7 @@ class BatchOperations:
         # Parse the Mutate response to print details about the entities that
         # were created by the request.
         suffix = "_result"
+        resource_names = list()
         for result in response.mutate_operation_responses:
             for field_descriptor, value in result._pb.ListFields():
                 if field_descriptor.name.endswith(suffix):
@@ -1444,6 +1446,8 @@ class BatchOperations:
                     f"Created a(n) {convert_snake_case_to_upper_case(name)} with "
                     f"{str(value).strip()}."
                 )
+                resource_names.append(str(value).strip())
+        return resource_names
 
 
 def add_negative_keywords(client, customer_id, shared_set_id, keywords, kw_type):
