@@ -1,25 +1,261 @@
 import os
 import re
-import resource
 import time
 import nltk
 import uuid
-import argparse
-import numpy as np
+import aiohttp
+import asyncio
 import pandas as pd
-from autoads.client import RestClient
 from bs4 import BeautifulSoup
 from nltk.corpus import stopwords
 from urllib.request import urlopen
-from base64 import urlsafe_b64decode
-from sentence_transformers import SentenceTransformer
-from sklearn.metrics.pairwise import cosine_similarity
+
+from autoads.client import AsyncRestClient, RestClient
 
 import sys
-from google.ads.googleads.client import GoogleAdsClient
 from google.ads.googleads.errors import GoogleAdsException
 
 
+async def get_keywords_from_api(email, api_key, seed_keywords, depth):
+    post_data = dict()
+
+    async def get_keywords(keyword, depth=depth, location='United States'):
+        async with aiohttp.ClientSession() as session:
+            client = AsyncRestClient(email, api_key,session)
+            post_data = dict()
+            post_data[len(post_data)] = dict(
+                keyword=keyword,
+                location_name=location,
+                language_name="English",
+                depth=depth,
+            )
+            post_data2 = dict()
+            post_data2[len(post_data2)] = dict(
+                keywords=[keyword],
+                location_name=location,
+                language_name="English",
+                depth=depth,
+            )
+            response = await client.post(
+                "/v3/dataforseo_labs/related_keywords/live", post_data)
+            response2 = await client.post(
+                "/v3/dataforseo_labs/keyword_ideas/live", post_data2)
+            response3 = await client.post(
+                "/v3/dataforseo_labs/keyword_suggestions/live", post_data)
+
+            return {
+                'related': response,
+                'ideas': response2,
+                'suggestions': response3
+            }
+
+    def extract_keywords(responses):
+        key_list = []
+        sources = []
+
+        # print(responses)
+
+        if responses['related']["status_code"] == 20000 and responses['related']['tasks'][0]['result'][0]['items']:
+            for x in range(len(responses['related']['tasks'][0]['result'][0]['items'])):
+                res = responses['related']['tasks'][0]['result'][0]['items'][x]['related_keywords']
+                if res is not None:
+                    key_list.extend(res)
+            print(f"{len(key_list)} related")
+            sources.extend(['related' for _ in range(len(key_list))])
+
+        if responses['ideas']["status_code"] == 20000 and responses['ideas']['tasks'][0]['result'][0]['items']:
+            # not good ideas
+            print(
+                f"{len(responses['ideas']['tasks'][0]['result'][0]['items'])} ideas")
+            for x in range(len(responses['ideas']['tasks'][0]['result'][0]['items'])):
+                res = responses['ideas']['tasks'][0]['result'][0]['items'][x]['keyword']
+                if res is not None:
+                    key_list.append(res)
+            sources.extend(['ideas' for _ in range(
+                len(responses['ideas']['tasks'][0]['result'][0]['items']))])
+
+        if responses['suggestions']["status_code"] == 20000 and responses['suggestions']['tasks'][0]['result'][0]['items']:
+            print(
+                f"{len(responses['suggestions']['tasks'][0]['result'][0]['items'])} suggestions")
+            for x in range(len(responses['suggestions']['tasks'][0]['result'][0]['items'])):
+                res = responses['suggestions']['tasks'][0]['result'][0]['items'][x]['keyword']
+                if res is not None:
+                    key_list.append(res)
+            sources.extend(['suggestions' for _ in range(
+                len(responses['suggestions']['tasks'][0]['result'][0]['items']))])
+        # else:
+        #     print("error. Code: %d Message: %s" %
+        #           (responses['all]["status_code"], responses['all']["status_message"]))
+        data = {
+            'Keywords': key_list,
+            'Sources': sources
+        }
+
+        return data
+
+    keyword_list = seed_keywords
+    # df_api = pd.DataFrame(columns=['Keywords', 'Keywords2', 'Sources'])
+    data_api = {'Keywords2': list(),
+                'Keywords': list(),
+                'sources': list()}
+
+    responses = list()
+    for keyword in keyword_list:
+        print(keyword)
+        responses.append(asyncio.create_task(get_keywords(keyword, depth)))
+
+    response_data = await asyncio.gather(*responses)
+
+    for data in response_data:
+        extracted = extract_keywords(data)
+        keywords2 = [keyword for _ in range(len(extracted['Keywords']))]
+        extracted['Keywords2'] = keywords2
+        data_api['Keywords2'].extend(keywords2)
+        data_api['Keywords'].extend(extracted['Keywords'])
+        data_api['sources'].extend(extracted['Sources'])
+
+    data_api = pd.DataFrame(data_api)
+    return data_api
+
+
+
+def get_keywords_from_scrape(urls,exclude,depth):
+
+    def add_spaces(text, thresh = 3, clean_n = False):
+        cleaned = ''
+        temp = [l.isupper() for l in text]
+        chk = 0
+        for i, s in enumerate(temp):
+            if s and i != 0 and (i - chk) > thresh:
+                cleaned += ' ' + ext[chk : i]
+                chk = i
+        for i, w in enumerate(cleaned):
+            if w != ' ':
+                cleaned = cleaned[i:]
+                break
+            else:
+                i+=1
+        if clean_n:
+            cleaned = cleaned.replace('\n', ' ')
+        return cleaned.replace('  ', ' ')
+    
+    def clean(text):
+        text = text.replace('*', '')
+        text = text.replace('\ufeff', '')
+        text = text.replace('\n', '')
+        text = text.replace('.', '')
+        text = text.replace('(', '')
+        text = text.replace(')', '')
+        text = text.replace('"', '')
+        text = text.replace('/', ' ')
+        text = text.replace('%', ' ')
+        text = text.replace('-', '')
+        text = text.replace('”', '')
+        text = text.replace('“', '')
+        text = text.replace('\'', '')
+        text = text.replace('!', '')
+        text = text.replace('?', '')
+        text = text.replace('&', '')
+        text = text.replace('+', '')
+        text = text.replace('$', '')
+        text = text.replace(',', '')
+        return text
+    
+    def _extract_(urls, depth = 1, return_urls = True, return_redirects = True, exclude = exclude):
+        print(urls)
+        exclude = exclude
+        scrape_urls = urls
+        resp = []
+        resp_urls = []
+        full_text = ''
+        depth = depth
+        for _ in range(depth):
+            temp_urls = []
+            for url in scrape_urls:
+                try:
+                    chk = 0
+                    for exc in exclude:
+                        if exc in url:
+                            # print(url)
+                            chk = 1
+                    
+                    if chk == 0:
+                        html = urlopen(url).read()
+                        # print(html)
+                        soup = BeautifulSoup(html, features="html.parser")
+                        
+                        for link in soup.find_all('a', attrs = {'href':re.compile('^/')}):
+                            uri = link.get('href')
+                            temp_urls.append(url + uri)
+                            # print(uri)
+                        
+                        for link in soup.find_all('a', attrs={'href': re.compile('^https://')}):
+                            uri = link.get('href')
+                            temp_urls.append(uri)
+                        
+                        for script in soup(['script', 'style']):
+                            script.extract()
+                        
+                        text = soup.get_text()
+                        
+                        lines = (line.strip() for line in text.splitlines())
+                        chunks = (phrase.strip() for line in lines for phrase in line.split("  "))
+                        text = '\n'.join(chunk for chunk in chunks if chunk)
+                        full_text += ' ' + text
+                        resp_urls.append(url)
+                        resp.append(1)
+                except:
+                    resp_urls.append(url)
+                    resp.append(0)
+                    continue
+        
+            scrape_urls = list(set(temp_urls))
+            
+        if return_urls:
+            if return_redirects:
+                return full_text, resp_urls, (resp_urls, resp)
+            else:
+                return full_text, resp_urls
+        elif return_redirects:
+            return full_text, (resp_urls, resp)
+        else:
+            return full_text
+    
+    fin_ngrams = []
+    new_urls = []
+    for url in urls:
+        if not 'http' in url:
+            if not 'www' in url:
+                new_urls.append(f'https://www.{url}')
+            else:
+                new_urls.append(f'https://{url}')
+    urls = new_urls
+    fin_ngrams = []
+    ext, _, _ = _extract_(urls, depth=depth)
+    # print(ext)
+    ext = add_spaces(ext, clean_n=True)
+        
+    nltk.download('stopwords')
+    stop = set(stopwords.words('english'))
+    text = clean(ext)
+    splt_text = text.split(' ')
+    nw_list = []
+    for t in splt_text:
+        if t not in stop and not t == '' and not t.isdigit() and len(t) > 1:
+            nw_list.append(t)
+            
+    ngrams = []
+    ngrams.extend(nltk.ngrams(nw_list, 3))
+    ngrams.extend(nltk.ngrams(nw_list, 4))
+    
+    for ngram in ngrams:
+        fin_ngrams.append(' '.join([ng for ng in ngram]))
+    data_scrape = {"Keywords":fin_ngrams}
+    data_scrape = pd.DataFrame(data_scrape)
+    return data_scrape
+
+
+
 def get_keywords_from_api_and_url(email,
                                   api_key,
                                   seed_keywords,
@@ -739,7 +975,7 @@ def get_keywords_metrics_google(client, customer_id, keywords):
 
         return resource_name
 
-    def _create_keyword_plan_ad_group_keywords(client, customer_id, plan_ad_group,keywords):
+    def _create_keyword_plan_ad_group_keywords(client, customer_id, plan_ad_group, keywords):
         """Adds keyword plan ad group keywords to the given keyword plan ad group.
 
         Args:
@@ -752,9 +988,9 @@ def get_keywords_metrics_google(client, customer_id, keywords):
             GoogleAdsException: If an error is returned from the API.
         """
         data = {
-            "Keywords":list(),
-            "keyword_id":list(),
-            "keyword_type":list(),
+            "Keywords": list(),
+            "keyword_id": list(),
+            "keyword_type": list(),
         }
         keyword_plan_ad_group_keyword_service = client.get_service(
             "KeywordPlanAdGroupKeywordService"
@@ -798,18 +1034,19 @@ def get_keywords_metrics_google(client, customer_id, keywords):
                 "Created keyword plan ad group keyword with resource name: "
                 f"{result.resource_name}"
             )
-        keywords_ids = [result.resource_name.split('/')[-1] for result in response.results]
+        keywords_ids = [result.resource_name.split(
+            '/')[-1] for result in response.results]
         data['keyword_id'] = keywords_ids
         return data
-    
+
     def _generate_forecast_metrics(client, customer_id, keyword_plan):
         data = {
-            "metrics_impression":list(),
-            "metrics_cpc":list(),
-            "metrics_clicks":list(),
-            "metrics_cost":list(),
-            "metrics_ctr":list(),
-            "keyword_id":list(),
+            "metrics_impression": list(),
+            "metrics_cpc": list(),
+            "metrics_clicks": list(),
+            "metrics_cost": list(),
+            "metrics_ctr": list(),
+            "keyword_id": list(),
         }
         keyword_plan_service = client.get_service("KeywordPlanService")
 
@@ -824,8 +1061,9 @@ def get_keywords_metrics_google(client, customer_id, keywords):
             data['metrics_cpc'].append(metrics.average_cpc)
             data['metrics_cost'].append(metrics.cost_micros)
             data['metrics_ctr'].append(metrics.ctr)
-            data['keyword_id'].append(forecast.keyword_plan_ad_group_keyword.split('/')[-1])
-        
+            data['keyword_id'].append(
+                forecast.keyword_plan_ad_group_keyword.split('/')[-1])
+
         return data
 
     keyword_plan = _create_keyword_plan(client, customer_id)
@@ -836,18 +1074,17 @@ def get_keywords_metrics_google(client, customer_id, keywords):
         client, customer_id, keyword_plan_campaign
     )
     keyword_data = _create_keyword_plan_ad_group_keywords(
-        client, customer_id, keyword_plan_ad_group,keywords
+        client, customer_id, keyword_plan_ad_group, keywords
     )
-    data = _generate_forecast_metrics(client,customer_id,keyword_plan)
-    
+    data = _generate_forecast_metrics(client, customer_id, keyword_plan)
+
     keyword_data = pd.DataFrame(keyword_data)
     data = pd.DataFrame(data)
 
-    data = data.merge(keyword_data,how='left',on='keyword_id')
+    data = data.merge(keyword_data, how='left', on='keyword_id')
     return data
 
 
-
 def make_request(keyword, email, api_key):
     client = RestClient(email, api_key)
     post_data = dict()
@@ -892,6 +1129,67 @@ def make_live_request(keyword, email, api_key, priority=2, depth=2, max_crawl_pa
               (response["status_code"], response["status_message"]))
 
 
+async def async_make_live_request(keyword, email, api_key, priority=2, depth=2, max_crawl_pages=2):
+
+    async with aiohttp.ClientSession() as session:
+        client = AsyncRestClient(email, api_key, session)
+        post_data = dict()
+        post_data[len(post_data)] = dict(
+            language_name="English",
+            location_name="United States",
+            keyword=keyword,
+            priority=priority,
+            depth=depth,
+            max_crawl_pages=max_crawl_pages,
+            tag=f"{keyword} google",
+        )
+        response = await client.post(
+            "/v3/serp/google/organic/live/advanced", post_data)
+        if response["status_code"] == 20000:
+            return response
+        else:
+            print("error. Code: %d Message: %s" %
+                  (response["status_code"], response["status_message"]))
+
+
+async def get_organic_description(email, api_key, keywords):
+
+    tasks = [
+        'organic',
+    ]
+    task_params = {
+        'organic': ['description'],
+    }
+
+    data = {
+        "organic_description": list(),
+        "keyword": keywords,
+    }
+
+    responses = list()
+    for keyword in keywords:
+        print(keyword)
+        responses.append(asyncio.create_task(
+            async_make_live_request(keyword, email, api_key, depth=5)))
+
+    response_data = await asyncio.gather(*responses)
+
+    for response in response_data:
+        if response['tasks'][0]['result']:
+            results = get_results(response, tasks, task_params)
+            organic_description = str([x.replace(",", "")
+                                      for x in results['organic_description']])
+            organic_description = organic_description.replace(
+                "[", "").replace("]", "")
+            data["organic_description"].append(organic_description)
+        else:
+            print("Failed to get organic description")
+            data["organic_description"].append(keyword)
+
+    data = pd.DataFrame(data)
+    return data
+
+
 def get_results(results, tasks, task_params):
     df = {
         'organic_url': list(),
