@@ -1,13 +1,15 @@
 import os
 import re
 import uuid
+import asyncio
 import numpy as np
 import pandas as pd
 from google.ads.googleads.client import GoogleAdsClient
 from sentence_transformers import SentenceTransformer
-from autoads.models import get_similarity_matrix,get_similarity_api
-from autoads.keywords import get_keywords_metrics,make_live_request,get_results
 from autoads.gads import get_existing_keywords
+from autoads.keywords import  get_organic_description
+from autoads.models import get_similarity_matrix,get_similarity_api
+
 
 email = 'weilbacherindustries@gmail.com' # email in data for seo
 api_key = '05f014a493983975' # api key for data for seo
@@ -45,42 +47,16 @@ df = df[df['similarity'] >= first_threshold]
 df = df[df['lf/mf/uf'].isin(allowed_funnels)].reset_index(drop=True)
 df = df.drop_duplicates("Keywords")
 
-#Second pass of similarity with page titles
-tasks = [
-    'organic',
-    # 'people_also_ask',
-]
-task_params = {
-    'organic': ['description'],
-    # 'people_also_ask': ['title'],
-}
 
-print("Getting Descriptions")
+print("Getting Organic Description")
 seed_keywords = df["Keywords2"].unique().tolist()
-for keyword in seed_keywords:
-    print(keyword)
-    response = make_live_request(keyword,email,api_key,depth=5)
-    if response['tasks'][0]['result']:
-        results = get_results(response,tasks,task_params)
-        organic_description = str([x.replace(",","") for x in results['organic_description']])
-        organic_description = organic_description.replace("[","").replace("]","")
-        df.loc[df["Keywords2"]== keyword,'seed_organic_description'] = str(organic_description)
-    else:
-        print("Failed to get organic description")
-        df.loc[df["Keywords2"]== keyword,'seed_organic_description'] =  keyword
+seed_organic_description = asyncio.run(get_organic_description(email,api_key,seed_keywords))
+seed_organic_description = seed_organic_description.rename(columns={'organic_description':'seed_organic_description'})
+df = df.merge(seed_organic_description,how='left',left_on='Keywords2',right_on='keyword')
 
 all_keywords = df["Keywords"].unique().tolist()
-for keyword in all_keywords:
-    print(keyword)
-    response = make_live_request(keyword,email,api_key,depth=5)
-    if response['tasks'][0]['result']:
-        results = get_results(response,tasks,task_params)
-        organic_description = str([x.replace(",","") for x in results['organic_description']])
-        organic_description = organic_description.replace("[","").replace("]","")
-        df.loc[df["Keywords"]== keyword,'organic_description'] = organic_description
-    else:
-        print("Failed to get organic description")
-        df.loc[df["Keywords"]== keyword,'organic_description'] = [keyword]  
+organic_description = asyncio.run(get_organic_description(email,api_key,all_keywords))
+df = df.merge(organic_description,how='left',left_on='Keywords',right_on='keyword')
 
 model = SentenceTransformer(similarity_model_path)
 
@@ -163,9 +139,6 @@ df = df.drop_duplicates('Keywords')
 
 if len(df) == 0:
     print("No new keywords to add")
-else:
-    print("Getting Keywords metrics")
-    # df = get_keywords_metrics(email,api_key,df,match_extract)
 
 df_existing.to_csv(save_path+'/df_existing.csv',index=False)
 df.to_csv(save_path+'/keywords_to_upload.csv',index=False)
