@@ -8,7 +8,7 @@ from autoads.keywords import get_keywords_metrics2
 from autoads.gads import BatchOperations, _handle_googleads_exception
 
 
-df_path = 'data/keywords_to_upload (1).csv' # specify keywords to uplod csv here
+df_path = 'data/keywords_to_upload.csv' # specify keywords to uplod csv here
 save_path = 'data'
 path = 'data/google-ads.yaml'
 customer_id = '8306215642' 
@@ -29,38 +29,37 @@ df_expand = df[~df['camp_id'].isna()].reset_index(drop=True)
 df_create = df[df['camp_id'].isna()].reset_index(drop=True)
 
 print("Getting keyword metrics")
-# df_create = get_keywords_metrics2(email, api_key, df_create, match_extract)
+df_create = get_keywords_metrics2(email, api_key, df_create, match_extract)
 
-# values = ['volume', 'cpc_exact', 'cpc_phrase']
-# df_create.loc[:, values] = df_create.loc[:, values] / 30
-# df_create['total_cost'] = 0.33 * df_create['volume'] * \
-#     ((df_create['cpc_exact'] + df_create['cpc_phrase']) / 2)
-# df_create['total_cost'] = df_create['total_cost'].cumsum()
-# df_create = df_create.sort_values(by=['total_cost'], na_position='last')
-# df_create = df_create[df_create['total_cost'] <= budget].reset_index(drop=True)
+values = ['volume', 'cpc_exact', 'cpc_phrase']
+df_create.loc[:, values] = df_create.loc[:, values] / 30
+df_create['total_cost'] = 0.33 * df_create['volume'] * \
+    ((df_create['cpc_exact'] + df_create['cpc_phrase']) / 2)
+df_create['total_cost'] = df_create['total_cost'].cumsum()
+df_create = df_create.sort_values(by=['total_cost'], na_position='last')
+df_create = df_create[df_create['total_cost'] <= budget].reset_index(drop=True)
 
-# spendings = df_create['total_cost'].iloc[-1]
-spendings = 100
+spendings = df_create['total_cost'].iloc[-1]
 
 batch_operations = BatchOperations(
     googleads_client,
     customer_id
 )
 
-# if len(df_expand) != 0:
-#     # code for expanding existing campaign
-#     print("Adding to existing campaign")
-#     for i, row in df_expand.iterrows():
-#         keyword = row['Keywords']
-#         campaign_id = str(int(row['camp_id']))
-#         batch_operations.create_adgroup(
-#             ad_group_name=keyword, campaign_id=campaign_id)
-#         batch_operations.create_keyword(keyword, kw_type='PHRASE')
-#         batch_operations.create_keyword(keyword, kw_type='EXACT')
+if len(df_expand) != 0:
+    # code for expanding existing campaign
+    print("Adding to existing campaign")
+    for i, row in df_expand.iterrows():
+        keyword = row['Keywords']
+        campaign_id = str(int(row['camp_id']))
+        batch_operations.create_adgroup(
+            ad_group_name=keyword, campaign_id=campaign_id)
+        batch_operations.create_keyword(keyword, kw_type='PHRASE')
+        batch_operations.create_keyword(keyword, kw_type='EXACT')
 
-#     print(f"{df_expand.shape[0]} campaigns will be expanded")
-# else:
-#     print("No keywords to add into existing campaigns")
+    print(f"{df_expand.shape[0]} campaigns will be expanded")
+else:
+    print("No keywords to add into existing campaigns")
 
 # Creating Campaigns
 if len(df_create) != 0:
